use crate::util::surrealdb::schemas;
use leptos::*;
use strsim::normalized_damerau_levenshtein;

#[derive(Clone)]
pub struct Time {
    pub minutes: u32,
    pub seconds: u32,
    pub milliseconds: u32,
}

impl Time {
    pub fn as_milliseconds(&self) -> u32 {
        self.minutes * 60 * 1000 + self.seconds * 1000 + self.milliseconds * 10
    }

    pub fn from_milliseconds_to_string(mut time: u32) -> String {
        let milliseconds = (time % 1000) / 10;
        time /= 1000;
        let seconds = time % 60;
        time /= 60;
        let minutes = time;

        format!("{minutes}:{seconds}:{milliseconds}")
    }

    pub fn from_milliseconds(mut time: u32) -> Self {
        let milliseconds = (time % 1000) / 10;
        time /= 1000;
        let seconds = time % 60;
        time /= 60;
        let minutes = time;

        Self {
            milliseconds,
            seconds,
            minutes,
        }
    }
}

pub fn sort_participants(
    participants: Vec<schemas::ParticipantSignal>,
    search: String,
) -> Vec<schemas::ParticipantSignal> {
    let mut filtered_sorted_list: Vec<(schemas::ParticipantSignal, f64)> = participants
        .into_iter()
        .map(|participant| {
            (
                participant.clone(),
                normalized_damerau_levenshtein(
                    &participant.value.get().name.to_lowercase(),
                    &search.to_lowercase(),
                ),
            )
        })
        .collect();

    filtered_sorted_list.sort_by(|a, b| {
        let (_, sim_score_a) = a;
        let (_, sim_score_b) = b;
        sim_score_b
            .partial_cmp(sim_score_a)
            .unwrap_or(std::cmp::Ordering::Equal)
    });

    filtered_sorted_list
        .into_iter()
        .map(|(item, _)| item)
        .collect()
}
